/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "export_utils.h"
#include "scripture.h"
#include "book.h"
#include "bible.h"
#include "usfm.h"
#include "usfmtools.h"
#include "project.h"
#include "notes_utils.h"
#include "date_time_utils.h"
#include "export_translation_notes.h"


void export_get_markers_to_skip (set<ustring>& markers_to_skip)
{
  markers_to_skip.insert ("mt");
  markers_to_skip.insert ("mt1");
  markers_to_skip.insert ("mt2");
  markers_to_skip.insert ("s");
}


void export_to_bibleworks (Configuration * configuration, GtkWidget * parent, Progress * progress)
/*
Exports a whole project to a file that is fit for being imported by the 
BibleWorks Version Database Compiler.
This done in this manner:
- If an \id line is found, use its ID to get the name of the book in BibleWorks.
- If an chapter tag if found, use its number.
- As soon as an ID is found, put the current chapter at one.
- If a verse is found, add its number.
- Text of tags known not to be printable is thrown out.
- Any other tag's text gets added to the text of the verse.
Yes, this is a bit rough, I know...
*/
{
  // Get the filename where the data to save to.
  GtkWidget *dialog;
  dialog = gtk_file_chooser_dialog_new ("Save to a file", GTK_WINDOW (parent),
    GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
    GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), configuration->export_to_bibleworks_filename.c_str ());
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  ustring filename;
  if (result == GTK_RESPONSE_ACCEPT) 
    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
  gtk_widget_destroy (dialog);  
  if (result == GTK_RESPONSE_ACCEPT) {
    // Store (new) filename.
    configuration->export_to_bibleworks_filename = filename;
    // Notebook shows progress bar.
    progress->start();
    // Start process.
    try
    {
      Project project (configuration->project);
      Usfm usfm (project.get_stylesheet());
      set<ustring> markers_to_skip;
      export_get_markers_to_skip (markers_to_skip);
      // Write to outputfile.
      WriteText wt (filename);
      // Open the Scripture and go through it.
      Scripture scripture (configuration->project);
      progress->set_iterate (0, 1, scripture.books.size());
      for (unsigned int bk = 0; bk < scripture.books.size(); bk++) {
        // Progress information.
        progress->iterate();
        if (progress->cancel) {
          progress->finish();
          return;
        }
        // Get the abbreviation to be used.
        ustring abbreviation;
        unsigned index = english_id_to_index (scripture.books[bk]);
        abbreviation = index_to_bibleworks_id (index);
        // Open the book
        Book book (scripture.paths[bk], false, "");
        // Start with chapter 1.
        ustring chapter = "1";
        // Go through the book.
        for (unsigned int i = 0; i < book.cleaned_lines.size(); i++) {
          ustring marker = usfm_extract_marker (book.cleaned_lines[i]);
          // Skip id line.
          if (usfm.is_identifier (marker))
            continue;
          // Skip running header.
          if (usfm.is_running_header (marker))
            continue;
          // Save chapter number.
          if (usfm.is_chapter_number(marker)) {
            chapter = number_in_string (book.cleaned_lines[i]);
            continue;
          }
          // If verse found, output new line, and its text.
          if (usfm.is_verse_number(marker)) {
            wt.text("\n");
            wt.text(abbreviation);
            wt.text(" ");
            wt.text(chapter);
            wt.text(":");
            wt.text(book.cleaned_lines[i]);
            continue;
          }
          // Skip list of know markers.
          if (markers_to_skip.find (marker) != markers_to_skip.end())
            continue;
          // Any other marker, known or unknown, output its text.
          wt.text(" ");
          wt.text(book.cleaned_lines[i]);
        }
      }
    }
    catch (exception & ex)
    {
      cerr << "Converting to BibleWorks: " << ex.what () << endl;
    }
  }
  // Notebook hides progress bar.
  progress->finish();
}


void export_translation_notes (const ustring& filename, ExportNotesFormat format, const vector<gint> & ids_to_display, bool export_all, GtkWidget * parent, Progress * progress)
{
  ExportTranslationNotes etn (filename, format, ids_to_display, export_all, parent, progress);
}
